@@ -1,30 +1,31 @@
+"""The module responsible for operating tgcf in live mode."""
 import logging
 
-from telethon import TelegramClient, events
+from telethon import events
 
-from tgcf.config import API_HASH, API_ID, CONFIG, SESSION
+from tgcf import config, const
+from tgcf.bot import (
+    forward_command_handler,
+    help_command_handler,
+    remove_command_handler,
+    start_command_handler,
+)
 from tgcf.plugins import apply_plugins
 from tgcf.utils import send_message
 
-from_to = {}
-
-KEEP_LAST_MANY = 10000
-
 existing_hashes = []
 
 _stored = {}
 
 
-for forward in CONFIG.forwards:
-    from_to[forward.source] = forward.dest
-
-
 class EventUid:
+    """The objects of this class uniquely identifies an event."""
+
     def __init__(self, event):
         self.chat_id = event.chat_id
         try:
             self.msg_id = event.id
-        except:
+        except:  # pylint: disable=bare-except
             self.msg_id = event.deleted_id
 
     def __str__(self):
@@ -38,26 +39,27 @@ class EventUid:
 
 
 async def new_message_handler(event):
+    """Process new incoming messages."""
     chat_id = event.chat_id
 
-    if chat_id not in from_to:
+    if chat_id not in config.from_to:
         return
     logging.info(f"New message received in {chat_id}")
     message = event.message
 
-    global _stored
+    global _stored  # pylint: disable=global-statement,invalid-name
 
     event_uid = EventUid(event)
 
     length = len(_stored)
-    exceeding = length - KEEP_LAST_MANY
+    exceeding = length - const.KEEP_LAST_MANY
 
     if exceeding > 0:
         for key in _stored:
             del _stored[key]
             break
 
-    to_send_to = from_to.get(chat_id)
+    to_send_to = config.from_to.get(chat_id)
 
     if to_send_to:
         if event_uid not in _stored:
@@ -74,11 +76,12 @@ async def new_message_handler(event):
 
 
 async def edited_message_handler(event):
+    """Handle message edits."""
     message = event.message
 
     chat_id = event.chat_id
 
-    if chat_id not in from_to:
+    if chat_id not in config.from_to:
         return
 
     logging.info(f"Message edited in {chat_id}")
@@ -94,22 +97,23 @@ async def edited_message_handler(event):
 
     if fwded_msgs:
         for msg in fwded_msgs:
-            if CONFIG.live.delete_on_edit == message.text:
+            if config.CONFIG.live.delete_on_edit == message.text:
                 await msg.delete()
                 await message.delete()
             else:
                 await msg.edit(message.text)
         return
 
-    to_send_to = from_to.get(event.chat_id)
+    to_send_to = config.from_to.get(event.chat_id)
 
     for recipient in to_send_to:
         await send_message(event.client, recipient, message)
 
 
 async def deleted_message_handler(event):
+    """Handle message deletes."""
     chat_id = event.chat_id
-    if chat_id not in from_to:
+    if chat_id not in config.from_to:
         return
 
     logging.info(f"Message deleted in {chat_id}")
@@ -123,6 +127,10 @@ async def deleted_message_handler(event):
 
 
 ALL_EVENTS = {
+    "bot_start": (start_command_handler, events.NewMessage(pattern="/start")),
+    "bot_forward": (forward_command_handler, events.NewMessage(pattern="/forward")),
+    "bot_remove": (remove_command_handler, events.NewMessage(pattern="/remove")),
+    "bot_help": (help_command_handler, events.NewMessage(pattern="/help")),
     "new": (new_message_handler, events.NewMessage()),
     "edited": (edited_message_handler, events.MessageEdited()),
     "deleted": (deleted_message_handler, events.MessageDeleted()),
@@ -130,11 +138,31 @@ ALL_EVENTS = {
 
 
 def start_sync():
-    client = TelegramClient(SESSION, API_ID, API_HASH)
+    """Start tgcf live sync."""
+    # pylint: disable= import-outside-toplevel
+    from telethon.sync import TelegramClient, functions, types
+
+    client = TelegramClient(config.SESSION, config.API_ID, config.API_HASH)
+    client.start(bot_token=config.BOT_TOKEN)
+    is_bot = client.is_bot()
+
     for key, val in ALL_EVENTS.items():
-        if CONFIG.live.delete_sync is False and key == "deleted":
+        if key.startswith("bot"):
+            if not is_bot:
+                continue
+        if config.CONFIG.live.delete_sync is False and key == "deleted":
             continue
         client.add_event_handler(*val)
         logging.info(f"Added event handler for {key}")
-    client.start()
+
+    if is_bot and const.REGISTER_COMMANDS:
+        client(
+            functions.bots.SetBotCommandsRequest(
+                commands=[
+                    types.BotCommand(command=key, description=value)
+                    for key, value in const.COMMANDS.items()
+                ]
+            )
+        )
+
     client.run_until_disconnected()
